package cn.ezhizao.project.business.controller;

import cn.ezhizao.framework.feign.SysFeignService;
import cn.ezhizao.framework.web.page.TableDataInfo;
import cn.ezhizao.project.business.domain.*;
import cn.ezhizao.project.business.service.*;
import cn.ezhizao.framework.aspectj.lang.annotation.Log;
import cn.ezhizao.framework.aspectj.lang.enums.BusinessType;
import cn.ezhizao.framework.web.controller.BaseController;
import cn.ezhizao.framework.web.domain.AjaxResult;
import cn.ezhizao.project.system.domain.SysDept;
import cn.ezhizao.project.system.domain.SysDictData;
import cn.ezhizao.project.system.domain.SysUser;
import cn.ezhizao.project.system.service.ISysDeptService;
import cn.ezhizao.project.system.service.ISysDictDataService;
import cn.ezhizao.project.system.service.ISysUserService;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;

import java.util.*;
import javax.annotation.Resource;

import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * mrp10201Controller
 *
 * @author ezhizao
 */
@RestController
@RequestMapping("/business/p2")
public class Mrp10201Controller extends BaseController {
    @Resource
    private IMrp10201Service mrp10201Service;

    @Resource
    private ISysUserService sysUserService;
    @Resource
    private ISysDeptService sysDeptService;
    @Resource
    private ISysDictDataService sysDictDataService;

    @Resource
    private IBizShaftCategoryService bizShaftCategoryService;

    @Resource
    private IBizProductionPlanDetailService productionPlanDetailService;

    @Resource
    private IBizPullP2TimeService pullP2TimeService;

    @Resource
    private IBizProductService productService;
    @Resource
    private IBizCompanyService companyService;

    @Resource
    private IBizTechnologicalProcessService technologicalProcessService;

    /**
     * 查询mrp10200列表
     */
    @GetMapping(value = "/test")
    private AjaxResult test() {
        return success();
    }

    /**
     * 获取mrp10200详细信息
     */
    @PreAuthorize("@ss.hasPermi('business:productionPlan:query')")
    @GetMapping(value = "/getP2Plan")
    public TableDataInfo getList(BizProductionPlanDetail data) {
        /*
         * 获取前端传递的参数信息
         * pullP2PlanDetails：如果为 true，说明点击了“同步P2中的生产计划按钮”；
         * 否则，则是“搜索”，根据已经拉取过来的，存储在【biz_production_plan_detail】中的数据进行搜索
         */

        // 先从新系统中的 biz_pull_p2_time 中获取最新数据
        BizPullP2Time pullDetail = new BizPullP2Time();
        pullDetail.setType("plan_detail");
        List<BizPullP2Time> pullDetails = pullP2TimeService.getList(pullDetail);
        /*
         * 如果 pullDetails 不为空，则获取最后一条数据，做为查询P2的生产计划明细的条件。
         * 这里有个细节，每次从P2中获取生产计划明细，不能都从头开始，否则随着数据量增加，将会重复拉取大量的数据，性能很差
         * 拉取数据后，存入新系统中，每次拉取前，将新系统中 biz_pull_p2_time 的最后一条数据记录的时间，做为条件传递过去
         * P2中，获取领料时间大于上一次拉取时间的新数据，再存入新系统中。
         */
        BizPullP2Time lastPullDetail = null;
        if (!pullDetails.isEmpty()) {
            int lastIndex = pullDetails.size() - 1;
            lastPullDetail = pullDetails.get(lastIndex);
        }

        if (data.getPullP2PlanDetails()) {
            // 当 pullP2PlanDetails 为 true 的时候，从P2数据库中拉取生产计划数据
            QueryWrapper<Mrp10201> queryWrapper = new QueryWrapper<>();
            if (lastPullDetail != null) {
                Date newCondition = lastPullDetail.getPullTime();
                /*
                 * 这里应该根据新系统中记录的最大的拉取日期
                 * lldate：P2中的领料日期，为了避免重复拉取，需要做条件判断。lldate必须大于上一次拉取时间
                 * llsta：领料状态，Y：领料，N：未领料，这里只需要拉取Y状态的领料
                 * lotbz：lotbz=‘Y’，表示正常领料；='G'：改料；='B'：补料
                 */
                queryWrapper.apply("lldate IS NOT NULL AND lldate > {0} AND llbz='Y' AND lotbz='Y'", newCondition);
            }
            List<Mrp10201> mrp10201List = mrp10201Service.getList(queryWrapper);
            // 如果从P2中拉取到数据，则需要保存到当前数据库中
            if (!mrp10201List.isEmpty()) {
                List<BizProductionPlanDetail> newPlanDetails = new ArrayList<>();
                for (Mrp10201 mrp10201:mrp10201List) {
                    // 判断下是否已经有该计划单，假设已有不能新增
                    String productionPlanNo = mrp10201.getDocno().trim();
                    Integer lineNum = mrp10201.getDocseq();
                    BizProductionPlanDetail newPlanDetail = productionPlanDetailService.query().eq("production_plan_no", productionPlanNo).eq("line_number", lineNum).one();
                    if (newPlanDetail == null) {
                        newPlanDetail = new BizProductionPlanDetail();
                    }
                    // 计划单号
                    newPlanDetail.setProductionPlanNo(mrp10201.getDocno());
                    // 根据拿到的计划单号，获取到对应的计划单id【暂时不用】
                    // 计划单序号
                    newPlanDetail.setLineNumber(mrp10201.getDocseq());
                    // 厂别（租户id）
                    newPlanDetail.setTenantId(Long.parseLong(mrp10201.getFacno()));

                    // 领料部门
                    newPlanDetail.setRequisitionDepartmentCode(mrp10201.getCusven());
                    SysDept requisitionDept = new SysDept();
                    requisitionDept.setDeptCode(newPlanDetail.getRequisitionDepartmentCode());
                    List<SysDept> sysDepts = new ArrayList<>();
                    if (requisitionDept.getDeptCode()!= null){
                        sysDepts = sysDeptService.selectDeptListByCode(requisitionDept);
                    }

                    // 根据领料部门编码，获取新系统中的部门id
                    if(!sysDepts.isEmpty()) {
                        newPlanDetail.setRequisitionDepartmentId(sysDepts.get(0).getDeptId());
                        newPlanDetail.setRequisitionDepartmentCode(sysDepts.get(0).getDeptCode());
                    }
                    System.out.println( sysDepts+"---------"+newPlanDetail.getRequisitionDepartmentId()+" ---------------------------测试-----------------------------" +newPlanDetail.getRequisitionDepartmentCode());
                    // 客户简称
                    newPlanDetail.setCompanyAlias(mrp10201.getKhname());
                    // 客户编码
                    newPlanDetail.setCompanyCode(mrp10201.getCust());
                    // 根据客户编码，获取新系统中的客户id【需要进一步实现】
                    List<BizCompany> companyCode = companyService.query().eq("company_code", newPlanDetail.getCompanyCode()).list();
                    if (!companyCode.isEmpty()){
                        newPlanDetail.setCompanyId(companyCode.get(0).getId());
                    }


                    // 商品编码
                    newPlanDetail.setProductCode(mrp10201.getCpart());
                    List<BizProduct> productCode = productService.query().eq("product_code", newPlanDetail.getProductCode()).list();
                    // 根据商品编码，获取新系统中的商品id
                    if (!productCode.isEmpty()){
                        newPlanDetail.setProductId(productCode.get(0).getId());
                    }
                    //根据商品版本获取工艺
                    newPlanDetail.setTechnologyVersion(mrp10201.getPrver());
                    List<BizTechnologicalProcess> technologicalProcessList = technologicalProcessService.query().eq("product_id", newPlanDetail.getProductId()).eq("technology_version", newPlanDetail.getTechnologyVersion()).list();
                    if (!technologicalProcessList.isEmpty()){
                        newPlanDetail.setTechnologicalProcessId(technologicalProcessList.get(0).getId());
                    }
                    // 图号
                    newPlanDetail.setDrawingNumber(mrp10201.getDraw());
                    // 商品规格
                    newPlanDetail.setSpecification(mrp10201.getSpec());

                    // 库位
                    newPlanDetail.setStorageLocation(mrp10201.getCprloc());

                    // 制造数量
                    newPlanDetail.setProductionQuantity(mrp10201.getZzqty());

                    // 投料量
                    newPlanDetail.setMaterialPickUpQuantity(mrp10201.getXeqty());

                    // 状态
                    String statusCode = mrp10201.getStatus();
                    newPlanDetail.setStatusCode(statusCode);
                    // 根据状态编码，获取新系统中的状态id【需要进一步实现】
                    SysDictData dictData = getDictData("production_plan_detail_status", statusCode);
                    newPlanDetail.setStatusId(dictData.getDictCode());

                    // 批号状态
                    String lotCodeStatusCode = mrp10201.getLotsta();
                    newPlanDetail.setLotCodeStatusCode(lotCodeStatusCode);
                    // 根据批号状态编码，获取新系统中的批号状态id【需要进一步实现】
                    dictData = getDictData("lot_code_status", lotCodeStatusCode);
                    newPlanDetail.setLotCodeStatusId(dictData.getDictCode());

                    // 批次类型
                    String lotTypeCode = mrp10201.getLotbz();
                    newPlanDetail.setLotTypeCode(lotTypeCode);
                    // 根据批次类型编码，获取新系统中的批次类型id【需要进一步实现】
                    dictData = getDictData("lot_type", lotTypeCode);
                    newPlanDetail.setLotTypeId(dictData.getDictCode());

                    // 生产类型
                    String productionTypeCode = mrp10201.getCptype();
                    newPlanDetail.setProductionTypeCode(productionTypeCode);
                    // 根据生产类型编码，获取新系统中的生产类型id【需要进一步实现】
                    dictData = getDictData("production_type", lotTypeCode);
                    newPlanDetail.setProductionTypeId(dictData.getDictCode());

                    // 备注
                    newPlanDetail.setRemark(mrp10201.getRemark());

                    // 领料人编码
                    newPlanDetail.setMaterialPickUpPersonCode(mrp10201.getEmno());
                    // 根据领料人编码，获取新系统中的领料人id【需要进一步实现】
                    SysUser userByCode = getUserByCode(newPlanDetail.getMaterialPickUpPersonCode());
                    if (userByCode!= null) {
                        newPlanDetail.setMaterialPickUpPersonId(userByCode.getUserId());
                    }

                    // BOM用量
                    newPlanDetail.setBomQuantity(mrp10201.getBase());
                    // 调度员
                    newPlanDetail.setDispatcherCode(mrp10201.getDdyno());
                    // 根据调度员编码，获取新系统中的调度员id【需要进一步实现】
                    userByCode = getUserByCode(newPlanDetail.getDispatcherCode());
                    if (userByCode!= null) {
                        newPlanDetail.setDispatcherId(userByCode.getUserId());
                    }

                    // 炉号
                    newPlanDetail.setFurnaceNumber(mrp10201.getLuhao());

                    // 销售员编码
                    newPlanDetail.setSalesmanCode(mrp10201.getXsyno());
                    // 根据销售员编码，获取新系统中的销售员id【需要进一步实现】
                    userByCode = getUserByCode(newPlanDetail.getSalesmanCode());
                    if (userByCode!= null) {
                        newPlanDetail.setSalesmanId(userByCode.getUserId());
                    }

                    // 产品描述
                    newPlanDetail.setProductDescription(mrp10201.getDescr());

                    // 轴类型
                    newPlanDetail.setShaftCategoryCode(mrp10201.getZtype());
                    // 根据轴类型编码，获取新系统中的轴类型id【需要进一步实现】
                    BizShaftCategory shaftCode = getShaftCode(newPlanDetail.getShaftCategoryCode());
                    if (shaftCode!= null) {
                        newPlanDetail.setShaftCategoryId(shaftCode.getId());
                    }
                    // 轴类型名称
                    newPlanDetail.setShaftCategoryName(mrp10201.getZtyna());

                    // 预完工日期
                    newPlanDetail.setExpectedCompletionDate(mrp10201.getYwdate());

                    // 保管员
                    newPlanDetail.setStockKeeperCode(mrp10201.getBgyno());
                    // 根据保管员编码，获取新系统中的保管员id【需要进一步实现】
                    userByCode = getUserByCode(newPlanDetail.getStockKeeperCode());
                    if (userByCode!= null) {
                        newPlanDetail.setStockKeeperId(userByCode.getUserId());
                    }

                    // 创建日期
                    newPlanDetail.setCreateTime(mrp10201.getDocdate());

                    // 投料日期
                    newPlanDetail.setFeedDate(mrp10201.getDddate());

                    // 领料日期
                    newPlanDetail.setMaterialPickUpDate(mrp10201.getLldate());

                    //下达日期
                    newPlanDetail.setIssueDate(mrp10201.getDddate());

                    // 总批数
                    newPlanDetail.setTotalLotNumber(mrp10201.getPhsl());

                    // 单批量
                    newPlanDetail.setOneLotQuantity(mrp10201.getPhsl1());

                    // 尾批量
                    newPlanDetail.setLastLotQuantity(mrp10201.getPhsl2());

                    // 主批号
                    newPlanDetail.setLotCode(mrp10201.getLotno());

                    // 对应的订单号
                    newPlanDetail.setSaleOrderCode(mrp10201.getDdno());

                    // 序号
                    newPlanDetail.setLineNumber(mrp10201.getDocseq());

                    // 是否外购生产
                    newPlanDetail.setIsPurchasedProduction(mrp10201.getZjbz());

                    // 尾批批号流水号
                    newPlanDetail.setLastLotSerialNumber(mrp10201.getPhzs());

                    // 是否领料
                    newPlanDetail.setIsPeekedUp(mrp10201.getLlsta());

                    // 领料日期
                    newPlanDetail.setMaterialPickUpDate(mrp10201.getLldate());

                    // 有批号否？
                    newPlanDetail.setHadLotCode(mrp10201.getPhsta());

                    // 备注
                    newPlanDetail.setRemark(mrp10201.getRemark());
                    newPlanDetails.add(newPlanDetail);
                    // 防止重复 根据计划单号 和 序号 排除重复项
//                    long count = productionPlanDetailService.query().eq("production_plan_id", newPlanDetail.getProductionPlanId()).eq("line_number", newPlanDetail.getLineNumber()).count();
//                    if(count == 0){
//                        newPlanDetails.add(newPlanDetail);
//                    }
                }
                if (!newPlanDetails.isEmpty()) {
//                  productionPlanDetailService.saveBatchByRawSql(newPlanDetails);
                    productionPlanDetailService.saveOrUpdateBatch(newPlanDetails);
                    // 当数据拉取成功，并将数据保存到 production_plan_detail 表中，则将 pull_p2_time 表中的 pull_time 属性置为当前时间
                    // 实例化 BizPullP2Time 对象
                    BizPullP2Time bizPullP2Time = new BizPullP2Time();
                    // 将当前时间赋值给 pullTime 属性
                    bizPullP2Time.setPullTime(new Date());
                    // 保存数据
                    pullP2TimeService.saveOrUpdate(bizPullP2Time);
                }
            }
        }

        // 通过 productionPlanDetailService 接口，获取 production_plan_detail 表中数据，并分页显示
        startPage();
        List<BizProductionPlanDetail> productionPlanDetails = productionPlanDetailService.getList(data);

        return getDataTable(productionPlanDetails);
    }

    /**
     * 新增区域配置
     */
    @PreAuthorize("@ss.hasPermi('business:productionPlan:save')")
    @Log(title = "区域配置", businessType = BusinessType.INSERT)
    @PostMapping("/save")
    @Transactional(rollbackFor = Exception.class)
    public AjaxResult save(@RequestBody Mrp10201 mrp10201) throws NoSuchFieldException, IllegalAccessException {
        setTenantId(mrp10201);
        mrp10201Service.saveOrUpdate(mrp10201);

        return success(mrp10201);
    }

    /**
     * 删除mrp10200
     */
    @PreAuthorize("@ss.hasPermi('business:productionPlan:remove')")
    @Log(title = "mrp10201", businessType = BusinessType.DELETE)
    @DeleteMapping("/{txcodes}")
    public AjaxResult remove(@PathVariable List<String> txcodes) {
        return toAjax(mrp10201Service.removeBatchByIds(txcodes));
    }

    /**
     * 根据字典类型和字典值，获取字典数据
     *
     * @param type  字典类型
     * @param value 字典值
     * @return 字典数据
     */
    private SysDictData getDictData(String type, String value) {
        SysDictData sysDictData = new SysDictData();
        sysDictData.setDictType(type);
        sysDictData.setDictValue(value);

        return sysDictDataService.getDictData(sysDictData);
    }

    /**
     * @param userName 用戶名
     * @return 用户信息
     */
    private SysUser getUserByCode(String userName) {
        return sysUserService.getUserByCode(userName.trim());
    }

    /**
     * 根据轴类型编码，获取新系统中的轴类型id【需要进一步实现】
     * @param code 轴类型编码
     * @return 轴类型
     */
    public BizShaftCategory getShaftCode(String code) {
        return bizShaftCategoryService.query().eq("sub_category_code", code).one();
    }
}
