package cn.ezhizao.project.business.controller;

import cn.ezhizao.framework.web.controller.BaseController;
import cn.ezhizao.framework.web.domain.AjaxResult;
import cn.ezhizao.project.business.domain.*;
import cn.ezhizao.project.business.domain.vo.OutsourceOrderDetailBalance;
import cn.ezhizao.project.business.service.*;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.concurrent.atomic.AtomicReference;
import java.util.stream.Collectors;

/**
 * 外协结算
 *
 * @author zx
 */
@RestController
@RequestMapping("/business/outsourceBalance")
public class OutsourceBalanceController extends BaseController {

    @Resource
    private IBizPullP2TimeService pullP2TimeService;

    @Resource
    private IBizMachineCenterService bizMachineCenterService;
    @Resource
    private IBizSupplierService bizSupplierService;
    @Resource
    private ISfc10100Service sfc10100Service;
    @Resource
    private IBizOutsourcedOrderService bizOutsourcedOrderService;
    @Resource
    private IBizOutsourcedOrderDetailService bizOutsourcedOrderDetailService;
    @Resource
    private IBizReturnReceiptService bizReturnReceiptService;
    @Resource
    private IBizReturnReceiptDetailService bizReturnReceiptDetailService;
    @Resource
    private IBizFurnaceNoInfoService bizFurnaceNoInfoService;
    @Resource
    private IInv10100Service inv10100Service;
    @Resource
    private IBizProductService bizProductService;
    @Resource
    private IBom30101Service bom30101Service;
    @Resource
    private IInc10100Service inc10100Service;
    @Resource
    private IBizOutsourcedOrderDetailProcessService bizOutsourcedOrderDetailProcessService;
    @Resource
    private IBizMaterialBaseService bizMaterialBaseService;
    @Resource
    private IBizMaterialBaseInfoService bizMaterialBaseInfoService;

    /**
     * 拉取 Sfc10100 表数据到 machine_center表
     *
     * @return 是否成功
     */
    private boolean pullP2Sfc10100() {
        BizPullP2Time pullDetail = new BizPullP2Time();
        // 加工中心
        pullDetail.setType("supplier");
        List<BizPullP2Time> pullDetails = pullP2TimeService.getList(pullDetail);
        BizPullP2Time lastPullDetail = null;
        if (pullDetails.size() > 0) {
            int lastIndex = pullDetails.size() - 1;
            lastPullDetail = pullDetails.get(lastIndex);
        }
        QueryWrapper<Sfc10100> queryWrapper = new QueryWrapper<>();
        if (lastPullDetail != null) {
            Date newCondition = lastPullDetail.getPullTime();
            /*
             * 这里应该根据新系统中记录的最大的拉取日期
             * modate：P2中的修改日期，为了避免重复拉取，需要做条件判断。modate必须大于上一次拉取时间
             */
            queryWrapper.apply("modate IS NOT NULL AND modate > {0}", newCondition);
        }
        List<Sfc10100> sfc10100List = sfc10100Service.getList(queryWrapper);
        List<BizSupplier> machineCenters = bizSupplierService.query().list();
        List<BizSupplier> insertMachineCenter = new ArrayList<>();
        List<BizSupplier> updateMachineCenter = new ArrayList<>();

        sfc10100List.forEach(l -> {
            // 判断之前是否有这个加工中心
            if (machineCenters.stream().anyMatch(v -> v.getCode().equals(l.getMccode().trim()))) {
                // 已插入加工中心
                BizSupplier oldItem = machineCenters.stream().filter(v -> v.getCode().equals(l.getMccode().trim())).findFirst().orElse(null);
                oldItem.setBySfc10100(l);
                updateMachineCenter.add(oldItem);
            } else {
                BizSupplier newItem = new BizSupplier(l);
                insertMachineCenter.add(newItem);
            }
        });

        if ((insertMachineCenter.isEmpty() || bizSupplierService.saveBatch(insertMachineCenter)) && (updateMachineCenter.isEmpty() || bizSupplierService.updateBatchById(updateMachineCenter))) {
            BizPullP2Time bizPullP2Time = new BizPullP2Time();
            // 将当前时间赋值给 pullTime 属性
            bizPullP2Time.setPullTime(new Date());
            bizPullP2Time.setType("supplier");
            // 保存数据
            pullP2TimeService.saveOrUpdate(bizPullP2Time);
            return true;
        }
        return false;
    }

    private boolean pullP2Inv10100() {
        BizPullP2Time pullDetail = new BizPullP2Time();
        // 加工中心
        pullDetail.setType("material_base");
        List<BizPullP2Time> pullDetails = pullP2TimeService.getList(pullDetail);
        BizPullP2Time lastPullDetail = null;
        if (pullDetails.size() > 0) {
            int lastIndex = pullDetails.size() - 1;
            lastPullDetail = pullDetails.get(lastIndex);
        }
        QueryWrapper<Inv10100> queryWrapper = new QueryWrapper<>();
        if (lastPullDetail != null) {
            Date newCondition = lastPullDetail.getPullTime();
            /*
             * 这里应该根据新系统中记录的最大的拉取日期
             * modate：P2中的修改日期，为了避免重复拉取，需要做条件判断。modate必须大于上一次拉取时间
             */
            queryWrapper.apply("modate IS NOT NULL AND modate > {0}", newCondition);
        }
        List<Inv10100> inv10100List = inv10100Service.getList(queryWrapper);
        List<BizMaterialBase> machineCenters = bizMaterialBaseService.query().list();
        List<BizMaterialBase> insertMachineCenter = new ArrayList<>();
        List<BizMaterialBase> updateMachineCenter = new ArrayList<>();

        inv10100List.forEach(l -> {
            // 判断之前是否有这个材料
            if (machineCenters.stream().anyMatch(v -> v.getMaterialCode().equals(l.getPart().trim()))) {
                // 已插入材料
                BizMaterialBase oldItem = machineCenters.stream().filter(v -> v.getMaterialCode().equals(l.getPart().trim())).findFirst().orElse(null);
                oldItem.setByInv10100(l);
                updateMachineCenter.add(oldItem);
            } else {
                BizMaterialBase newItem = new BizMaterialBase(l);
                insertMachineCenter.add(newItem);
            }
        });

        if ((insertMachineCenter.isEmpty() || bizMaterialBaseService.saveBatch(insertMachineCenter)) && (updateMachineCenter.isEmpty() || bizMaterialBaseService.updateBatchById(updateMachineCenter))) {
            BizPullP2Time bizPullP2Time = new BizPullP2Time();
            // 将当前时间赋值给 pullTime 属性
            bizPullP2Time.setPullTime(new Date());
            bizPullP2Time.setType("material_base");
            // 保存数据
            pullP2TimeService.saveOrUpdate(bizPullP2Time);
            return true;
        }
        return false;
    }

    private boolean pullP2Bom30101() {
        BizPullP2Time pullDetail = new BizPullP2Time();
        // 加工中心
        pullDetail.setType("material_base_info");
        List<BizPullP2Time> pullDetails = pullP2TimeService.getList(pullDetail);
        BizPullP2Time lastPullDetail = null;
        if (pullDetails.size() > 0) {
            int lastIndex = pullDetails.size() - 1;
            lastPullDetail = pullDetails.get(lastIndex);
        }
        QueryWrapper<Bom30101> queryWrapper = new QueryWrapper<>();
        if (lastPullDetail != null) {
            Date newCondition = lastPullDetail.getPullTime();
            /*
             * 这里应该根据新系统中记录的最大的拉取日期
             * modate：P2中的修改日期，为了避免重复拉取，需要做条件判断。modate必须大于上一次拉取时间
             */
            queryWrapper.apply("modate IS NOT NULL AND modate > {0}", newCondition);
        }
        List<Bom30101> bom30101List = bom30101Service.getList(queryWrapper);
        List<BizMaterialBaseInfo> machineCenters = bizMaterialBaseInfoService.query().list();
        List<BizMaterialBaseInfo> insertMachineCenter = new ArrayList<>();
        List<BizMaterialBaseInfo> updateMachineCenter = new ArrayList<>();

        bom30101List.forEach(l -> {
            // 判断之前是否有这个材料
            if (machineCenters.stream().anyMatch(v -> v.getMaterialCode().equals(l.getPart().trim()) && v.getProductCode().equals(l.getCpart().trim()))) {
                // 已插入材料
                BizMaterialBaseInfo oldItem = machineCenters.stream().filter(v -> v.getMaterialCode().equals(l.getPart().trim()) && v.getProductCode().equals(l.getCpart().trim())).findFirst().orElse(null);
                oldItem.setByBom30101(l);
                updateMachineCenter.add(oldItem);
            } else {
                BizMaterialBaseInfo newItem = new BizMaterialBaseInfo(l);
                insertMachineCenter.add(newItem);
            }
        });

        if ((insertMachineCenter.isEmpty() || bizMaterialBaseInfoService.saveBatch(insertMachineCenter)) && (updateMachineCenter.isEmpty() || bizMaterialBaseInfoService.updateBatchById(updateMachineCenter))) {
            BizPullP2Time bizPullP2Time = new BizPullP2Time();
            // 将当前时间赋值给 pullTime 属性
            bizPullP2Time.setPullTime(new Date());
            bizPullP2Time.setType("material_base_info");
            // 保存数据
            pullP2TimeService.saveOrUpdate(bizPullP2Time);
            return true;
        }
        return false;
    }
    /**
     * 外协结算结果
     *
     * @return 返回值
     */
    @GetMapping(value = "/settlement")
    public AjaxResult settlement() {
        // 获取所有已收回的外协发出单，是否已经结算过目前没有标识方法。目前不考虑
        List<BizOutsourcedOrderDetail> orderDetails = bizOutsourcedOrderDetailService.query().exists("(select 1 from biz_return_receipt_detail t where t.outsource_detail_id = biz_outsourced_order_detail.id and t.deleted = 0 and status = 1)").list();
        List<BizReturnReceiptDetail> returnReceiptDetails = bizReturnReceiptDetailService.query().eq("status", 1).list();
        List<BizFurnaceNoInfo> furnaceNoInfos = bizFurnaceNoInfoService.query().in("daywork_id", orderDetails.isEmpty() ? Collections.singletonList(0L) : orderDetails.stream().map(BizOutsourcedOrderDetail::getDayworkId).collect(Collectors.toList())).list();

        boolean pullResult = pullP2Sfc10100();
        boolean pullMaterialResult = pullP2Inv10100();
        boolean pullMaterialInfoResult = pullP2Bom30101();
        // 获取产品材料信息
//        QueryWrapper<Inv10100> queryWrapper = new QueryWrapper<>();
//        queryWrapper.in("rtrim(part)", furnaceNoInfos.isEmpty() ? Collections.singletonList("0") : furnaceNoInfos.stream().map(BizFurnaceNoInfo::getRawMaterialCode).collect(Collectors.toList()));
//        List<Inv10100> inv10100s = inv10100Service.getList(queryWrapper);
        List<BizMaterialBase> materialBases = bizMaterialBaseService.query().in("material_code", furnaceNoInfos.isEmpty() ? Collections.singletonList("0") : furnaceNoInfos.stream().map(BizFurnaceNoInfo::getRawMaterialCode).collect(Collectors.toList())).list();
        List<BizProduct> products = bizProductService.query().in("id", orderDetails.isEmpty() ? Collections.singletonList(0L) : orderDetails.stream().map(BizOutsourcedOrderDetail::getProductId).collect(Collectors.toList())).list();
//        QueryWrapper<Bom30101> bom30101QueryWrapper = new QueryWrapper<>();
//        bom30101QueryWrapper.in("rtrim(cpart)", products.isEmpty() ? Collections.singletonList("0") : products.stream().map(BizProduct::getProductCode).collect(Collectors.toList()));
//        List<Bom30101> bom30101s = bom30101Service.getList(bom30101QueryWrapper);
        List<BizMaterialBaseInfo> materialBaseInfos = bizMaterialBaseInfoService.query().in("product_code", products.isEmpty() ? Collections.singletonList("0") : products.stream().map(BizProduct::getProductCode).collect(Collectors.toList())).list();
        QueryWrapper<Inc10100> inc10100QueryWrapper = new QueryWrapper<>();
        inc10100QueryWrapper.in("rtrim(part)", products.isEmpty() ? Collections.singletonList("0") : products.stream().map(BizProduct::getProductCode).collect(Collectors.toList()));
        List<Inc10100> inc10100List = inc10100Service.getList(inc10100QueryWrapper);
        List<BizOutsourcedOrderDetailProcess> processes = bizOutsourcedOrderDetailProcessService.query().in("detail_id", orderDetails.isEmpty() ? Collections.singletonList(0L) : orderDetails.stream().map(BizOutsourcedOrderDetail::getId).collect(Collectors.toList())).list();

        List<BizSupplier> suppliers = bizSupplierService.query().list();
        List<OutsourceOrderDetailBalance> balances = new ArrayList<>();
        orderDetails.forEach(l -> {
            l.setProcesses(processes.stream().filter(v -> v.getDetailId().equals(l.getId())).collect(Collectors.toList()));
            OutsourceOrderDetailBalance balance = new OutsourceOrderDetailBalance();
            balance.setFormCode(l.getFormCode());
            balance.setFormDate(l.getFormDate());
            balance.setSupplierCode(l.getSupplierCode());
            balance.setSupplierName(l.getSupplierName());
            BizSupplier supplier = suppliers.stream().filter(v -> v.getId().equals(l.getSupplierId())).findFirst().orElse(null);
            List<BizReturnReceiptDetail> currentReturn = returnReceiptDetails.stream().filter(v -> v.getOutsourceDetailId().equals(l.getId())).collect(Collectors.toList());
            BizFurnaceNoInfo furnaceNoInfo = furnaceNoInfos.stream().filter(v -> v.getDayworkId().equals(l.getDayworkId())).findFirst().orElse(new BizFurnaceNoInfo());
            BizMaterialBase itemInv10100 = materialBases.stream().filter(v -> v.getMaterialCode().trim().equals(furnaceNoInfo.getRawMaterialCode())).findFirst().orElse(null);
            BizProduct product = products.stream().filter(v -> v.getId().equals(l.getProductId())).findFirst().orElse(new BizProduct());
            BizMaterialBaseInfo itemBom30101 = materialBaseInfos.stream().filter(v -> v.getProductCode().trim().equals(product.getProductCode())).findFirst().orElse(new BizMaterialBaseInfo());
            Inc10100 itemInc10100 = inc10100List.stream().filter(v -> v.getPart().trim().equals(product.getProductCode())).findFirst().orElse(new Inc10100());
            balance.setProductCode(product.getProductCode());
            balance.setProductDescription(product.getDescription());
            balance.setProcessName(processes.stream().map(BizOutsourcedOrderDetailProcess::getProcessAlias).collect(Collectors.joining()));
            balance.setProductName(product.getDrawingNumber());
            balance.setProductNum(l.getProductNum());
            balance.setLotCode(l.getLotCode());
            balance.setSupplierCode(supplier != null ? supplier.getCode() : "");
            balance.setTaxRatio(supplier != null ? supplier.getTaxRatio() : null);
            // 料损数量
            // 99984与99024无料损数量 【不扣料损】
            // 其他外协厂[料损数量]计算公式=发出数量 * (1 - 料损率 / 1000) - 合格数量，结果取整 【料损率从外协厂基础配置表中读】
            if (l.getSupplierCode().equals("99984") || l.getSupplierCode().equals("99024")) {
                balance.setMaterialLoss(BigDecimal.ZERO);
            } else {
                BigDecimal auditNum = BigDecimal.valueOf(currentReturn.stream().mapToInt(BizReturnReceiptDetail::getAuditNum).sum());
                BigDecimal materialLoss = BigDecimal.valueOf(l.getProductNum()).multiply(supplier.getLossLimit().divide(BigDecimal.valueOf(-1000), 6, RoundingMode.HALF_UP).add(BigDecimal.ONE)).subtract(auditNum).setScale(0, RoundingMode.HALF_UP);
                balance.setMaterialLoss(materialLoss);
            }
            // 实际投料量
            // 用原料编码在inv10100取type时，值为2对应半成品料。
            // 值为4对应原材料，此时，取xzcode，值是11和12时，分别对应盘圆料与直条料
            // 如果是盘圆料/直条料/半成品料
            // [实际投料量]计算公式=实际投料直径 / 2 * 实际投料直径 / 2 * 3.14 * 原料密度 / 1000 * (产品长度  + 公差长度【bom30101.gccd（材料基本信息表）按cpart产品编码取，生产计划随时变动结算时实时获取】) / 原材料利用率 / 10000，结果保留四位小数
            // 其他情况下 [实际投料量]直接取inc10100.stqty
            if (itemInv10100 != null && ((itemInv10100.getType().trim().equals("4") && (itemInv10100.getXzcode().trim().equals("11") || itemInv10100.getXzcode().trim().equals("12"))) || itemInv10100.getType().trim().equals("2"))) {
                // 直径
                BigDecimal diameter = furnaceNoInfo.getDiameter().equals("") ? BigDecimal.ZERO : BigDecimal.valueOf(Double.parseDouble(furnaceNoInfo.getDiameter()));
                // 半径
                BigDecimal r = diameter.divide(BigDecimal.valueOf(2), 6, RoundingMode.HALF_UP);
                // 原料密度
                BigDecimal density = itemInv10100.getDensity();
                // 成品长度
                BigDecimal length = product.getLenght() != null ? product.getLenght() : BigDecimal.ZERO;
                // 公差长度
                BigDecimal gccd = itemBom30101.getGccd();
                // 原材料利用率
                BigDecimal utrate = itemBom30101.getUtrate();
                balance.setActualFeeding(r.multiply(r).multiply(BigDecimal.valueOf(3.14)).multiply(density).divide(BigDecimal.valueOf(1000), 6, RoundingMode.HALF_UP).multiply(length.add(gccd)).divide(utrate, 9, RoundingMode.HALF_UP).divide(BigDecimal.valueOf(10000), 4, RoundingMode.HALF_UP));
            } else {
                BigDecimal actualFeeding = itemInc10100.getStqty();
                balance.setActualFeeding(actualFeeding);
            }
            // 废品计算类别
            balance.setWastePriceCategory(supplier != null ? supplier.getWastePriceCategory() : "");
            // 结算方式
            balance.setSettlementType(supplier == null ? "" : supplier.getSettlementType().equals("1") ? "按支结算" : supplier.getSettlementType().equals("2") ? "按炉结算" :"");
            // 运费
            // 运费单价 * 发出箱数 保留4位小数
            balance.setFreightPrice(supplier != null ? supplier.getFreightDeductUnitPrice().multiply(BigDecimal.valueOf(l.getNewCarrierCount())).setScale(4, RoundingMode.HALF_UP) : BigDecimal.ZERO);
            if (supplier != null && supplier.getSurfaceTreatment().equals("N")) {
                // 废品加工单价
                // 废品计算类别 '11' 且没得表面处理 废品加工单价 = round(实际投料量 * 原料最近采购单价[inv10100.zjunprc实时变更],4)
                if (balance.getWastePriceCategory().equals("11")) {
                    balance.setWastePrice(balance.getActualFeeding().multiply(itemInv10100.getZjunprc()).setScale(4, RoundingMode.HALF_UP));
                }
                // 废品计算类别为 '12' 且没得表面处理  且外协厂编码='99997' 且 （产品编码）='911110028' 废品加工单价 = round(原料最近采购单价 ,4)
                if (balance.getWastePriceCategory().equals("12") && supplier.getCode().equals("99997") && product.getProductCode().equals("911110028")) {
                    balance.setWastePrice(itemInv10100.getZjunprc().setScale(4, RoundingMode.HALF_UP));
                }
                // 废品计算类别为 '12' 且没得表面处理  且外协厂编码='99980' 且 （产品编码）='911100538' 废品加工单价 = round(原料最近采购单价 ,4)
                if (balance.getWastePriceCategory().equals("12") && supplier.getCode().equals("99980") && product.getProductCode().equals("911100538")) {
                    balance.setWastePrice(itemInv10100.getZjunprc().setScale(4, RoundingMode.HALF_UP));
                }
                // 废品计算类别是'24' 且没有表面处理 废品加工单价 round(加工单价[外协厂结算员填写的],4)
                if (balance.getWastePriceCategory().equals("24")) {
                    balance.setWastePrice(supplier != null ? supplier.getFreightPrice().setScale(4, RoundingMode.HALF_UP) : BigDecimal.ZERO);
                }
                if (balance.getWastePrice() == null) {
                    balance.setWastePrice(BigDecimal.ZERO);
                }
                // 料扣金额 = round(料损数量 * 废品加工单价 * 120/100.00 * 1.13【*税率】,4)//(1 + 税率（取自sfc10100.taxsl） /100.00 ),2)
                BigDecimal materialCost = balance.getMaterialLoss().multiply(balance.getWastePrice()).multiply(BigDecimal.valueOf(120).divide(BigDecimal.valueOf(100), 6, RoundingMode.HALF_UP)).multiply(BigDecimal.ONE.add(BigDecimal.valueOf(supplier.getTaxRatio()).divide(BigDecimal.valueOf(100), 2, RoundingMode.HALF_UP))).setScale(4, RoundingMode.HALF_UP);
                balance.setMaterialLoss(materialCost);
                if (supplier.getSettlementType().equals("1")) {
                    // 按支结算
                    AtomicReference<BigDecimal> planAmount = new AtomicReference<>(BigDecimal.ZERO);
                    if(supplier.getCode().equals("99024")) {
                        l.getProcesses().forEach(t -> {
                            if (t.getProcessCode().equals("99000")) {
                                // 应得金额=round(合格数量 * 加工单价 * (1.00 + 税率 /100.00 )    ,4)
                                planAmount.set(planAmount.get().add(BigDecimal.valueOf(t.getQualifiedNum()).multiply(supplier.getFreightPrice()).multiply(BigDecimal.ONE.add(BigDecimal.valueOf(supplier.getTaxRatio()).divide(BigDecimal.valueOf(100), 2, RoundingMode.HALF_UP))).setScale(4, RoundingMode.HALF_UP)));
                            } else {
                                // 应得金额=round(发出数量 * 加工单价 * (1.00 + 税率 /100.00 )    ,4)
                                planAmount.set(planAmount.get().add(BigDecimal.valueOf(l.getProductNum()).multiply(supplier.getFreightPrice()).multiply(BigDecimal.ONE.add(BigDecimal.valueOf(supplier.getTaxRatio()).divide(BigDecimal.valueOf(100), 2, RoundingMode.HALF_UP))).setScale(4, RoundingMode.HALF_UP)));

                            }
                        });
                    } else {
                        // 应得金额=round(合格数量 * 加工单价 * (1.00 + 税率 /100.00 )    ,4)
                        l.getProcesses().forEach(t -> {
                            planAmount.set(planAmount.get().add(BigDecimal.valueOf(t.getQualifiedNum()).multiply(supplier.getFreightPrice()).multiply(BigDecimal.ONE.add(BigDecimal.valueOf(supplier.getTaxRatio()).divide(BigDecimal.valueOf(100), 2, RoundingMode.HALF_UP))).setScale(4, RoundingMode.HALF_UP)));
                        });
                    }
                    balance.setPlanAmount(planAmount.get());
                } else if (supplier.getSettlementType().equals("2")) {
                    // 按炉结算
                    // 暂时先放着
                }
            } else if (supplier != null && supplier.getSurfaceTreatment().equals("Y")) {
                // 废品加工单价
                if (balance.getWastePriceCategory().equals("21")) {
                    // 废品计算类别 21 废品加工单价 = round(产品单价(inc10100.unprc) -  加工单价 ,4)
                    balance.setWastePrice(itemInc10100.getUnprc().subtract(supplier.getFreightPrice()).setScale(4, RoundingMode.HALF_UP));
                } else if (balance.getWastePriceCategory().equals("22")) {
                    // 废品计算类别 22 废品加工单价 = round(加工单价  * 2 ,4)
                    balance.setWastePrice(supplier.getFreightPrice().multiply(BigDecimal.valueOf(2)).setScale(4, RoundingMode.HALF_UP));
                } else if (balance.getWastePriceCategory().equals("23")) {
                    // 废品加工单价 = round(加工单价  * 3 ,4)
                    balance.setWastePrice(supplier.getFreightPrice().multiply(BigDecimal.valueOf(3)).setScale(4, RoundingMode.HALF_UP));
                }
                if (balance.getWastePrice() == null || balance.getWastePrice().compareTo(BigDecimal.ZERO) < 0) {
                    balance.setWastePrice(BigDecimal.ZERO);
                }
                // 料扣金额
                // 料扣金额 = round(料损数量 * 废品加工单价 * 1.13【*税率】 ,4) // (1 + 税率 /100.00 ),2)
                BigDecimal materialCost = balance.getMaterialLoss().multiply(balance.getWastePrice()).multiply(BigDecimal.ONE.add(BigDecimal.valueOf(supplier.getTaxRatio()).divide(BigDecimal.valueOf(100), 2, RoundingMode.HALF_UP))).setScale(4, RoundingMode.HALF_UP);
                balance.setMaterialLoss(materialCost);
                // 结算金额
                if (supplier.getSettlementType().equals("1")) {
                    // 按支结算
                    // 外协厂编码='99024'
                    AtomicReference<BigDecimal> planAmount = new AtomicReference<>(BigDecimal.ZERO);
                    if(supplier.getCode().equals("99024")) {
                        // 应得金额=round(发出数量 * 加工单价 * (1.00 + 税率 /100.00 ),4)
                        l.getProcesses().forEach(t -> {
                            planAmount.set(BigDecimal.valueOf(l.getProductNum()).multiply(supplier.getFreightPrice()).multiply(BigDecimal.ONE.add(BigDecimal.valueOf(supplier.getTaxRatio()).divide(BigDecimal.valueOf(100), 2, RoundingMode.HALF_UP))).setScale(4, RoundingMode.HALF_UP));
                        });
                    } else {
                        // 应得金额=round(合格数量 * 加工单价 * (1.00 + 税率 /100.00 ),4)
                        l.getProcesses().forEach(t -> {
                            planAmount.set(BigDecimal.valueOf(t.getQualifiedNum()).multiply(supplier.getFreightPrice()).multiply(BigDecimal.ONE.add(BigDecimal.valueOf(supplier.getTaxRatio()).divide(BigDecimal.valueOf(100), 2, RoundingMode.HALF_UP))).setScale(4, RoundingMode.HALF_UP));
                        });
                    }
                    balance.setPlanAmount(planAmount.get());
                } else if (supplier.getSettlementType().equals("2")) {
                    // 按炉结算
                }
            }
            balance.setFreightPrice(supplier != null ? supplier.getFreightDeductUnitPrice().multiply(BigDecimal.valueOf(l.getNewCarrierCount())).setScale(4, RoundingMode.HALF_UP) : null);
            balances.add(balance);
        });
        return success(balances);
    }
}
