package cn.ezhizao.project.business.controller;

import cn.ezhizao.framework.web.controller.BaseController;
import cn.ezhizao.framework.web.domain.AjaxResult;
import cn.ezhizao.framework.web.page.TableDataInfo;
import cn.ezhizao.project.business.domain.*;
import cn.ezhizao.project.business.service.*;
import cn.ezhizao.project.system.domain.SysDictData;
import cn.ezhizao.project.system.domain.SysUser;
import cn.ezhizao.project.system.service.ISysDeptService;
import cn.ezhizao.project.system.service.ISysDictDataService;
import cn.ezhizao.project.system.service.ISysUserService;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * sfc10300Controller
 *
 * @author ezhizao_zx
 */
@RestController
@RequestMapping("/business/sfc10300")
public class Sfc10300Controller  extends BaseController {

    @Resource
    private IBizPullP2TimeService pullP2TimeService;
    @Resource
    private ISfc10300Service sfc10300Service;
    @Resource
    private IBizProcessService processService;
    @Resource
    private ISysUserService sysUserService;
    @Resource
    private ISysDeptService sysDeptService;
    @Resource
    private ISysDictDataService sysDictDataService;

    @Resource
    private IBizShaftCategoryService bizShaftCategoryService;

    /**
     * 获取sfc10300详细信息
     * 获取 sfc10300 信息传到 biz_product
     */
    @PreAuthorize("@ss.hasPermi('business:process:sync')")
    @GetMapping(value = "/getP2Process")
    public AjaxResult getP2Process() {
        // 先从新系统中的 biz_pull_p2_time 中获取最新数据
        BizPullP2Time pullDetail = new BizPullP2Time();
        pullDetail.setType("process");
        List<BizPullP2Time> pullDetails = pullP2TimeService.getList(pullDetail);
        /*
         * 如果 pullDetails 不为空，则获取最后一条数据，做为查询P2的生产计划明细的条件。
         * 这里有个细节，每次从P2中获取生产计划明细，不能都从头开始，否则随着数据量增加，将会重复拉取大量的数据，性能很差
         * 拉取数据后，存入新系统中，每次拉取前，将新系统中 biz_pull_p2_time 的最后一条数据记录的时间，做为条件传递过去
         * P2中，获取领料时间大于上一次拉取时间的新数据，再存入新系统中。
         */
        BizPullP2Time lastPullDetail = null;
        if (pullDetails.size() > 0) {
            int lastIndex = pullDetails.size() - 1;
            lastPullDetail = pullDetails.get(lastIndex);
        }

        QueryWrapper<Sfc10300> queryWrapper = new QueryWrapper<>();
        if (lastPullDetail != null) {
            Date newCondition = lastPullDetail.getPullTime();
            /*
             * 这里应该根据新系统中记录的最大的拉取日期
             * modate：P2中的修改日期，为了避免重复拉取，需要做条件判断。modate必须大于上一次拉取时间
             */
            queryWrapper.apply("modate IS NOT NULL  AND modate > {0}", newCondition);
        }
        List<Sfc10300> sfc10300List = sfc10300Service.getList(queryWrapper);
        List<BizProcess> processes = processService.query().list();
        List<BizProcess> newProcess = new ArrayList<>();
        List<BizProcess> changeProcess = new ArrayList<>();
        List<SysUser> userList = sysUserService.getList();
        SysDictData conditions = new SysDictData();
        conditions.setDictType("process_status");
        List<SysDictData> dictList = sysDictDataService.selectDictDataList(conditions);
        List<BizShaftCategory> categories = bizShaftCategoryService.query().list();
        sfc10300List.forEach(l -> {
            if (processes.stream().anyMatch(v -> v.getProcessCode().equals(l.getPrcode().trim()))) {
                // 已插入产品
                BizProcess old = processes.stream().filter(v -> v.getProcessCode().equals(l.getPrcode().trim())).findFirst().orElse(null);
                if (old != null) {
                    old.setProcessKey(l.getFinkey());
                    old.setProcessAlias(l.getPrna());
                    old.setType(l.getType());
                    old.setStatus(l.getStatus());
                    changeProcess.add(old);
                }
            } else {
                // 新产品
                BizProcess bizProcess = new BizProcess();
                bizProcess.setProcessCode(l.getPrcode());

                bizProcess.setProcessKey(l.getFinkey());
                bizProcess.setProcessAlias(l.getPrna());
                bizProcess.setType(l.getType());
                bizProcess.setStatus(l.getStatus());
                newProcess.add(bizProcess);
            }
        });
        if ((newProcess.isEmpty() || processService.saveBatch(newProcess)) && (changeProcess.isEmpty() || processService.updateBatchById(changeProcess))) {
            BizPullP2Time bizPullP2Time = new BizPullP2Time();
            // 将当前时间赋值给 pullTime 属性
            bizPullP2Time.setPullTime(new Date());
            bizPullP2Time.setType("process");
            // 保存数据
            pullP2TimeService.saveOrUpdate(bizPullP2Time);
            return toAjax(true);
        }
        return toAjax(false);
    }


    @GetMapping(value = "/getP2ProcessWrbz/{type}")
    public TableDataInfo getP2ProcessWrbz (@PathVariable("type") String type) {
        List <Sfc10300> sfc10300List = sfc10300Service.getProcessWrbz();
        return getDataTable( sfc10300List);
    }
    /**
     * 根据字典类型和字典值，获取字典数据
     *
     * @param type  字典类型
     * @param value 字典值
     * @return 字典数据
     */
    private SysDictData getDictData(String type, String value) {
        SysDictData sysDictData = new SysDictData();
        sysDictData.setDictType(type);
        sysDictData.setDictValue(value);

        return sysDictDataService.getDictData(sysDictData);
    }

    /**
     * @param userName 用戶名
     * @return 用户信息
     */
    private SysUser getUserByCode(String userName) {
        return sysUserService.getUserByCode(userName.trim());
    }

    /**
     * 根据轴类型编码，获取新系统中的轴类型id【需要进一步实现】
     * @param code 轴类型编码
     * @return 轴类型
     */
    public BizShaftCategory getShaftCode(String code) {
        return bizShaftCategoryService.query().eq("sub_category_code", code).one();
    }

}