package cn.ezhizao.project.business.domain;

import cn.ezhizao.framework.aspectj.lang.annotation.Excel;
import cn.ezhizao.framework.web.domain.BaseEntity;
import com.baomidou.mybatisplus.annotation.TableName;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;

import java.math.BigDecimal;

/**
 * 外协商主表
 *
 * @author ezhizao
 * date    2024-04-24
 */
@Data
@EqualsAndHashCode(callSuper = true)
@TableName(value = "biz_supplier")
public class BizSupplier extends BaseEntity {
    private static final long serialVersionUID = 1L;

    @ApiModelProperty(value = "租户（厂别）id")
    private Long tenantId;

    @Excel(name = "公司编码")
    @ApiModelProperty(value = "公司编码")
    private String code;

    @Excel(name = "公司名称")
    @ApiModelProperty(value = "公司名称")
    private String name;

    @Excel(name = "公司别名")
    @ApiModelProperty(value = "公司别名")
    private String alias;

    @Excel(name = "助记码")
    @ApiModelProperty(value = "助记码")
    private String mnemonicCode;

    @Excel(name = "地区")
    @ApiModelProperty(value = "地区")
    private String region;

    @Excel(name = "送货方式")
    @ApiModelProperty(value = "送货方式：0：自提；1：物流")
    private String deliveryMethod;

    @Excel(name = "结算方式")
    @ApiModelProperty(value = "0：无结算方式，1：按支结算，2：按炉结算")
    private String settlementType;

    @ApiModelProperty("加工单价")
    private BigDecimal freightPrice;

    @Excel(name = "损耗上限")
    @ApiModelProperty(value = "损耗上限")
    private BigDecimal lossLimit;

    @Excel(name = "备注")
    @ApiModelProperty(value = "备注")
    private String remark;

    /** 状态 NA外协单位可用,CL外协单位不用 */
    @Excel(name = "状态 NA外协单位可用,CL外协单位不用")
    @ApiModelProperty(value = "状态 NA外协单位可用,CL外协单位不用")
    private String status;

    /** 废品单价类别 */
    @Excel(name = "废品单价类别")
    @ApiModelProperty(value = "废品单价类别")
    private String wastePriceCategory;

    /** Y有表面处理按售价-工序单价计算N无表面处理工序 */
    @Excel(name = "Y有表面处理按售价-工序单价计算N无表面处理工序")
    @ApiModelProperty(value = "Y有表面处理按售价-工序单价计算N无表面处理工序")
    private String surfaceTreatment;

    /** 运费扣款单价 */
    @Excel(name = "运费扣款单价")
    @ApiModelProperty(value = "运费扣款单价")
    private BigDecimal freightDeductUnitPrice;

    /** 税率 */
    @Excel(name = "税率")
    @ApiModelProperty(value = "税率")
    private Integer taxRatio;

    @ApiModelProperty("外协加工类别：11：热处理12：机加工13：表面处理14：电镀15：三方选别")
    private String outsourceType;

    @ApiModelProperty("是否免检 免检为N 质检为Y")
    private String isQualityChecking;

    public BizSupplier(Sfc10100 l) {
        this.code = l.getMccode().trim();
        this.name = l.getMcname().trim();
        this.mnemonicCode = l.getFinkey().trim();
        this.lossLimit = l.getLsl();
        this.settlementType = l.getJsfs().trim().equals("N") ? "1": l.getJsfs().trim().equals("Y") ? "2" : "0" ;
        this.status = l.getStatus() != null ? l.getStatus().trim() : null;
        this.freightDeductUnitPrice = l.getYfprc();
        this.surfaceTreatment = l.getBmcl().trim();
        this.wastePriceCategory = l.getFptype();
        this.taxRatio = l.getTaxsl();
        this.outsourceType = l.getWxtype();
        this.isQualityChecking = l.getMjtype();
    }

    public BizSupplier() {
        super();
    }

    public void setBySfc10100(Sfc10100 l) {
        this.code = l.getMccode().trim();
        this.name = l.getMcname().trim();
        this.mnemonicCode = l.getFinkey().trim();
        this.lossLimit = l.getLsl();
        this.settlementType = l.getJsfs().trim().equals("N") ? "1": l.getJsfs().trim().equals("Y") ? "2" : "0" ;
        this.status = l.getStatus() != null ? l.getStatus().trim() : null;
        this.freightDeductUnitPrice = l.getYfprc();
        this.surfaceTreatment = l.getBmcl().trim();
        this.wastePriceCategory = l.getFptype();
        this.taxRatio = l.getTaxsl();
        this.outsourceType = l.getWxtype();
        this.isQualityChecking = l.getMjtype();
    }
}
